''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
 # Project  :SC16IS752
 # Describe :UART0 communicates with UART1
 #			 GPIO0-GPIO3 levels flip every second
 #           read GPIO4-GPIO7 levels every second
 # Experimental Platform :Raspberry Pi + Serial Extension Module
 # Hardware Connection : Header
 #	I2C_Address:	A0 -> 3V3		IRQ_PIN:	IRQ -> P5
 #			        A2 -> 3V3					
 # Library Version :Python 3.73
 # Author		   :Christian
 # Web Site		   :www.seengreat.com
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
#!/usr/bin/python
# -*- coding:utf-8 -*-
import time
import serial

IN = 0
OUT = 1
LOW = 0
HIGH = 1
NUM_MAXBUF = 4
DIR_MAXSIZE = 60
PIN_NUM_START = 625

class Sc16is752():
    def __init__(self):
        self.ser0 = serial.Serial("/dev/ttySC0",115200,timeout=1)
        self.ser1 = serial.Serial("/dev/ttySC1",115200,timeout=1)
        self.gpio_enable()

    def gpio_export(self, pin_num):
        fd = open("/sys/class/gpio/export",'w')
        fd.write('%d' % pin_num)
        print( 'Export: GPIO%d' % (pin_num - PIN_NUM_START))
    
    def gpio_unexport(self, pin_num):
        fd = open("/sys/class/gpio/unexport",'w')
        fd.write('%d' % pin_num)
        print( 'Export: GPIO%d' % (pin_num - PIN_NUM_START))
        fd.close()

    def gpio_direction(self, pin_num, pin_dir):
        path = "/sys/class/gpio/gpio%d/direction" % pin_num
        fd = open(path,'w')
        if pin_dir == IN:
            fd.write('in')
            print('GPIO%d:intput' % (pin_num - PIN_NUM_START))
        elif pin_dir:
            fd.write('out')
            print('GPIO%d:Output' % (pin_num - PIN_NUM_START))
        fd.close()
    
    # Pin=0 corresponds to GPIO0 on the board,and so on 
    def gpio_read(self, pin_num):
        pin_num += PIN_NUM_START
        path = "/sys/class/gpio/gpio%d/value" % pin_num
        fd = open(path,'r')
        res = fd.read()
        fd.close()
        return int(res)

    # Pin=0 corresponds to GPIO0 on the board,and so on     
    def gpio_write(self, pin_num, value):
        pin_num += PIN_NUM_START
        path = "/sys/class/gpio/gpio%d/value" % pin_num
        fd = open(path,'w')
        if value == LOW:
            fd.write('0')
        else:
            fd.write('1')
        fd.close()

    def gpio_mode(self, pin_num, mode):
        pin_num += PIN_NUM_START
        self.gpio_direction(pin_num, mode)
       
    def gpio_enable(self):
        for num in range(8):
            self.gpio_export(num + PIN_NUM_START)
        
    def gpio_disable(self):
        for num in range(8):
            self.gpio_unexport(num + PIN_NUM_START)
#*****************************************************
# This demo codes sets GPIO0~3 as output,GPIO4~7 as
# input,communicates with each other between ttySC0 and 
# ttySC1.and then inverts the output state of GPIO0~3 
# and read the input state of GPIO4~7 every second, 
# Here is the wiring definition:
# TXDA-----RXDB
# RXDA-----TXDB
# if user want the hardware flow control function,
# and add the "rtscts=True" in the serial.Serial function
# CTSA-----RTSB
# RTSA-----CTSB
# **************************************************/

if __name__ == '__main__':
    ser_expan = Sc16is752()
    for num in range(4):
        ser_expan.gpio_mode(num, OUT)
    for num in range(4,8):
        ser_expan.gpio_mode(num, IN)
    out_val = 0    
    try:
        while True:
            print("\r\nStart sending data\r\n")
            ser_expan.ser0.write("Hello word".encode())
            time.sleep(0.01)
            ser_expan.ser1.write("www.seengreat.com".encode())
            time.sleep(0.05)
            
            print("uart0 received data is:")
            while ser_expan.ser0.inWaiting() > 0:
                uart0_rev = ser_expan.ser0.read(ser_expan.ser0.inWaiting())
                print(uart0_rev)
            print("uart1 received data is:")
            while ser_expan.ser1.inWaiting() > 0:
                uart1_rev = ser_expan.ser1.read(ser_expan.ser1.inWaiting())
                print(uart1_rev)

            for num in range(4):
                ser_expan.gpio_write(num, out_val)
            for num in range(4,8):
                val = ser_expan.gpio_read(num)
                print("GPIO",num,"is",val)
            if out_val == 0:
                out_val = 1
            else:
                out_val = 0
            time.sleep(1)
    except KeyboardInterrupt:
        ser_expan.ser0.close()
        ser_expan.ser1.close()
        ser_expan.gpio_disable()
        print("program exit!")
